import { logger } from "../log.js";
const tgapi = require("node-telegram-bot-api")
  , EventEmitter = require("events").EventEmitter
  , util = require("util");

class tg {
  constructor(options) {
    EventEmitter.call(this);
    this.options = options || {};
    this.token = options.token || null;
    this.options.polling = options.polling || true;
    this.client = new tgapi(this.options.token, this.options);

    this.server = {
      channel: new Map(),
      user: new Map()
    };

    this.client.on("message", msg => {
      if (!this.server.user.has(msg.from.first_name))
        this.server.user.set(msg.from.username || msg.from.first_name, {
          nick: msg.from.first_name,
          username: msg.from.username,
          id: msg.from.id
        });

      if (msg.date >= (~~(Date.now() / 1000) - 10) && msg.text !== undefined)
        this.emit("data", ["message", this.reply(msg)]);
    });
  }
  send(id, msg) {
    this.client.sendMessage(id, msg, { parse_mode: "HTML" });
  }
  reply(tmp) {
    return {
      type: "tg",
      network: "Telegram",
      channel: tmp.chat.title,
      channelid: tmp.chat.id,
      user: {
        nick: tmp.from.first_name,
        username: tmp.from.username,
        hostname: tmp.from.id
      },
      message: tmp.text,
      time: tmp.date,
      raw: tmp,
      reply: msg => this.send(tmp.chat.id, this.format(msg)),
      replyAction: msg => this.send(tmp.chat.id, this.format(`Uwe ${msg}`)),
      replyNotice: msg => this.send(tmp.chat.id, this.format(msg)),
      _user: this.server.user
    };
  }
  format(msg) {
    return ""+msg
      .replace("<", "&lt;")
      .replace(">", "&gt;")
      .replace(/\[b\](.*?)\[\/b\]/g, "<b>$1</b>") // bold
      .replace(/\[i\](.*?)\[\/i\]/g, "<i>$1</i>") // italic
    ;
  }
}

module.exports = tg;
util.inherits(tg, EventEmitter);